//!
//!
use crate::repositories::{
  bank::models::Bank as _Bank, member::models::Member as _Member,
  member::schema::MemberState as _MemberState, member_class::models::MemberClass as _MemberClass,
  member_game_setting::models::MemberGameSetting as _MemberGameSetting,
  member_level::models::MemberLevel as _MemberLevel,
  member_settlement_setting::models::MemberSettlementSetting as _MemberSettlementSetting,
  site::models::Site as _Site,
};
use diesel::deserialize::QueryableByName;

///
#[derive(PartialEq, PartialOrd, Debug, Clone)]
pub struct MemberModel {
  ///
  pub id: uuid::Uuid,
  ///
  pub site: _Site,
  ///
  pub member_class: _MemberClass,
  ///
  pub member_level: _MemberLevel,
  ///
  pub bank_account: Option<MemberBankAccountModel>,
  ///
  pub username: String,
  ///
  pub password: String,
  ///
  pub nickname: String,
  ///
  pub parent_member: Option<_Member>,
  ///
  pub child_member_count: i64,
  ///
  pub mobile_phone_number: Option<String>,
  ///
  pub state: _MemberState,
  ///
  pub state_changed_at: Option<i64>,
  ///
  pub member_game_setting: Option<_MemberGameSetting>,
  ///
  pub member_settlement_setting: Option<_MemberSettlementSetting>,
  ///
  pub last_signined_ip: Option<String>,
  ///
  pub last_signined_at: Option<i64>,
  ///
  pub created_at: i64,
  ///
  pub updated_at: i64,
  ///
  pub deleted_at: Option<i64>,
}

impl QueryableByName<diesel::pg::Pg> for MemberModel {
  fn build<R: diesel::row::NamedRow<diesel::pg::Pg>>(row: &R) -> diesel::deserialize::Result<Self> {
    let parent_member = match row
      .get::<diesel::sql_types::Nullable<diesel::sql_types::Uuid>, Option<uuid::Uuid>>(
        "m_parent_member_id",
      )? {
      Some(_) => Some(_Member {
        id: row.get("_m_id")?,
        site_id: row.get("_m_site_id")?,
        member_class_id: row.get("_m_member_class_id")?,
        member_level_id: row.get("_m_member_level_id")?,
        username: row.get("_m_username")?,
        password: row.get("_m_password")?,
        nickname: row.get("_m_nickname")?,
        mobile_phone_number: row.get("_m_mobile_phone_number")?,
        state: row.get("_m_state")?,
        state_changed_at: row.get("_m_state_changed_at")?,
        parent_member_id: row.get("_m_parent_member_id")?,
        child_member_count: row.get("_m_child_member_count")?,
        last_signined_ip: row.get("_m_last_signined_ip")?,
        last_signined_at: row.get("_m_last_signined_at")?,
        created_at: row.get("_m_created_at")?,
        updated_at: row.get("_m_updated_at")?,
        deleted_at: row.get("_m_deleted_at")?,
      }),
      None => None,
    };

    let bank = match row
      .get::<diesel::sql_types::Nullable<diesel::sql_types::Uuid>, Option<uuid::Uuid>>("_b_id")?
    {
      Some(_) => Some(_Bank {
        id: row.get("_b_id")?,
        name: row.get("_b_name")?,
        sort_order: row.get("_b_sort_order")?,
        show: row.get("_b_show")?,
        can_use: row.get("_b_can_use")?,
        memo: row.get("_b_memo")?,
        created_at: row.get("_b_created_at")?,
        updated_at: row.get("_b_updated_at")?,
      }),
      None => None,
    };

    let member_bank_account = match bank {
      Some(bank) => Some(MemberBankAccountModel {
        id: row.get("mba_id")?,
        member_id: row.get("mba_member_id")?,
        bank,
        name: row.get("mba_name")?,
        account_number: row.get("mba_account_number")?,
        exchange_password: row.get("mba_exchange_password")?,
        memo: row.get("mba_memo")?,
        created_at: row.get("mba_created_at")?,
        updated_at: row.get("mba_updated_at")?,
      }),
      None => None,
    };

    let member_game_setting = match row
      .get::<diesel::sql_types::Nullable<diesel::sql_types::Uuid>, Option<uuid::Uuid>>("mgs_id")?
    {
      Some(_) => Some(_MemberGameSetting {
        id: row.get("mgs_id")?,
        member_id: row.get("mgs_member_id")?,
        can_bet_casino: row.get("mgs_can_bet_casino")?,
        can_bet_slot: row.get("mgs_can_bet_slot")?,
        can_bet_powerball: row.get("mgs_can_bet_powerball")?,
        can_bet_powerladder: row.get("mgs_can_bet_powerladder")?,
        can_bet_eos: row.get("mgs_can_bet_eos")?,
        can_bet_bogglepowerball: row.get("mgs_can_bet_bogglepowerball")?,
        created_at: row.get("mgs_created_at")?,
        updated_at: row.get("mgs_updated_at")?,
      }),
      None => None,
    };

    let member_settlement_setting = match row
      .get::<diesel::sql_types::Nullable<diesel::sql_types::Uuid>, Option<uuid::Uuid>>("mss_id")?
    {
      Some(_) => Some(_MemberSettlementSetting {
        id: row.get("mss_id")?,
        member_id: row.get("mss_member_id")?,
        can_exchange: row.get("mss_can_exchange")?,
        can_first_charge_comp: row.get("mss_can_first_charge_comp")?,
        can_per_charge_comp: row.get("mss_can_per_charge_comp")?,
        manual_payment_type_for_partner: row.get("mss_manual_payment_type_for_partner")?,
        settlement_type: row.get("mss_settlement_type")?,
        rate_casino: row.get("mss_rate_casino")?,
        rate_casino_loosing: row.get("mss_rate_casino_loosing")?,
        rate_casino_bacara: row.get("mss_rate_casino_bacara")?,
        rate_casino_roulette: row.get("mss_rate_casino_roulette")?,
        rate_casino_dragon_tiger: row.get("mss_rate_casino_dragon_tiger")?,
        rate_slot: row.get("mss_rate_slot")?,
        rate_slot_loosing: row.get("mss_rate_slot_loosing")?,
        rate_powerball_single: row.get("mss_rate_powerball_single")?,
        rate_powerball_combo: row.get("mss_rate_powerball_combo")?,
        rate_powerladder_single: row.get("mss_rate_powerladder_single")?,
        rate_powerladder_combo: row.get("mss_rate_powerladder_combo")?,
        rate_eos_single: row.get("mss_rate_eos_single")?,
        rate_eos_combo: row.get("mss_rate_eos_combo")?,
        rate_bogglepowerball_single: row.get("mss_rate_bogglepowerball_single")?,
        rate_bogglepowerball_combo: row.get("mss_rate_bogglepowerball_combo")?,
        created_at: row.get("mss_created_at")?,
        updated_at: row.get("mss_updated_at")?,
      }),
      None => None,
    };

    Ok(MemberModel {
      id: row.get("m_id")?,
      site: _Site {
        id: row.get("s_id")?,
        url: row.get("s_url")?,
        name: row.get("s_name")?,
        path: row.get("s_path")?,
        show: row.get("s_show")?,
        can_use: row.get("s_can_use")?,
        memo: row.get("s_memo")?,
        expires_at: row.get("s_expires_at")?,
        created_at: row.get("s_created_at")?,
        updated_at: row.get("s_updated_at")?,
      },
      member_class: _MemberClass {
        id: row.get("mc_id")?,
        parent_id: row.get("mc_parent_id")?,
        name: row.get("mc_name")?,
        show: row.get("mc_show")?,
        created_at: row.get("mc_created_at")?,
        updated_at: row.get("mc_updated_at")?,
      },
      member_level: _MemberLevel {
        id: row.get("ml_id")?,
        name: row.get("ml_name")?,
        show: row.get("ml_show")?,
        sort_order: row.get("ml_sort_order")?,
        created_at: row.get("ml_created_at")?,
        updated_at: row.get("ml_updated_at")?,
      },
      username: row.get("m_username")?,
      password: row.get("m_password")?,
      nickname: row.get("m_nickname")?,
      mobile_phone_number: row.get("m_mobile_phone_number")?,
      state: row.get("m_state")?,
      state_changed_at: row.get("m_state_changed_at")?,
      bank_account: member_bank_account,
      member_game_setting,
      member_settlement_setting,
      parent_member,
      child_member_count: row.get("m_child_member_count")?,
      last_signined_ip: row.get("m_last_signined_ip")?,
      last_signined_at: row.get("m_last_signined_at")?,
      created_at: row.get("m_created_at")?,
      updated_at: row.get("m_updated_at")?,
      deleted_at: row.get("m_deleted_at")?,
    })
  }
}

#[derive(Eq, Hash, PartialEq, PartialOrd, Debug, Clone)]
pub struct MemberBankAccountModel {
  ///
  pub id: uuid::Uuid,
  ///
  pub member_id: uuid::Uuid,
  ///
  pub bank: _Bank,
  ///
  pub name: String,
  ///
  pub account_number: String,
  ///
  pub exchange_password: String,
  ///
  pub memo: Option<String>,
  ///
  pub created_at: i64,
  ///
  pub updated_at: i64,
}
